@@ -37,10 +37,8 @@
 #include <libmissioncontrol/mission-control.h>
 #include <libmissioncontrol/mc-account-monitor.h>
 #include <libtelepathy/tp-constants.h>
-#include <libtelepathy/tp-helpers.h>
 
 #include <libempathy/gossip-debug.h>
-#include <libempathy/gossip-paths.h>
 #include <libempathy/gossip-utils.h>
 #include <libempathy-gtk/gossip-ui-utils.h>
 
@@ -186,12 +184,12 @@ accounts_dialog_setup (GossipAccountsDialog *dialog)
 
 		status = mission_control_get_connection_status (dialog->mc, account, NULL);
 
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter,
-				    COL_NAME, name,
-				    COL_STATUS, status,
-				    COL_ACCOUNT_POINTER, account,
-				    -1);
+		gtk_list_store_insert_with_values (store, &iter,
+						   -1,
+						   COL_NAME, name,
+						   COL_STATUS, status,
+						   COL_ACCOUNT_POINTER, account,
+						   -1);
 
 		accounts_dialog_status_changed_cb (dialog->mc,
 						   status,
@@ -305,15 +303,12 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 
 	if (account) {
 		McProfile *profile;
-		GdkPixbuf *pixbuf;
-
-		pixbuf = gossip_pixbuf_from_account (account, GTK_ICON_SIZE_DIALOG);
-		gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->image_type), pixbuf);
-		if (pixbuf) {
-			g_object_unref (pixbuf);
-		}
 
 		profile = mc_account_get_profile (account);
+		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->image_type),
+					      mc_profile_get_icon_name (profile),
+					      GTK_ICON_SIZE_DIALOG);
+		
 
 		gtk_label_set_text (GTK_LABEL (dialog->label_type),
 				    mc_profile_get_display_name (profile));
@@ -412,6 +407,7 @@ accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn    *tree_column,
 					GossipAccountsDialog *dialog)
 {
 	McAccount                 *account;
+	const gchar               *icon_name;
 	GdkPixbuf                 *pixbuf;
 	TelepathyConnectionStatus  status;
 
@@ -420,7 +416,8 @@ accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn    *tree_column,
 			    COL_ACCOUNT_POINTER, &account,
 			    -1);
 
-	pixbuf = gossip_pixbuf_from_account (account, GTK_ICON_SIZE_BUTTON);
+	icon_name = gossip_icon_name_from_account (account);
+	pixbuf = gossip_pixbuf_from_icon_name (icon_name, GTK_ICON_SIZE_BUTTON);
 
 	if (pixbuf) {
 		if (status == TP_CONN_STATUS_DISCONNECTED ||
@@ -592,12 +589,12 @@ accounts_dialog_add_account (GossipAccountsDialog *dialog,
 
 	gossip_debug (DEBUG_DOMAIN, "Adding new account: %s", name);
 
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter,
-			    COL_NAME, name,
-			    COL_STATUS, status,
-			    COL_ACCOUNT_POINTER, account,
-			    -1);
+	gtk_list_store_insert_with_values (store, &iter,
+					   -1,
+					   COL_NAME, name,
+					   COL_STATUS, status,
+					   COL_ACCOUNT_POINTER, account,
+					   -1);
 }
 
 static void
@@ -677,6 +674,10 @@ accounts_dialog_status_changed_cb (MissionControl                  *mc,
 	model = gtk_tree_view_get_model (view);
 	account = mc_account_lookup (unique_name);
 
+	gossip_debug (DEBUG_DOMAIN, "Status changed for account %s: "
+		      "status=%d presence=%d",
+		      unique_name, status, presence);
+
 	for (ok = gtk_tree_model_get_iter_first (model, &iter);
 	     ok;
 	     ok = gtk_tree_model_iter_next (model, &iter)) {
@@ -738,6 +739,8 @@ accounts_dialog_status_changed_cb (MissionControl                  *mc,
 		g_source_remove (dialog->connecting_id);
 		dialog->connecting_id = 0;
 	}
+
+	gtk_widget_show (dialog->window);
 }
 
 static void          
@@ -940,7 +943,7 @@ accounts_dialog_destroy_cb (GtkWidget            *widget,
 }
 
 GtkWidget *
-gossip_accounts_dialog_show (void)
+gossip_accounts_dialog_show (GtkWindow *parent)
 {
 	static GossipAccountsDialog *dialog = NULL;
 	GladeXML                    *glade;
@@ -1004,7 +1007,7 @@ gossip_accounts_dialog_show (void)
 	gtk_widget_show (dialog->combobox_profile);
 
 	/* Set up signalling */
-	dialog->mc = mission_control_new (tp_get_bus ());
+	dialog->mc = gossip_mission_control_new ();
 	dialog->monitor = mc_account_monitor_new ();
 
 	/* FIXME: connect account-enabled/disabled too */
@@ -1020,11 +1023,15 @@ gossip_accounts_dialog_show (void)
 
 	accounts_dialog_model_setup (dialog);
 	accounts_dialog_setup (dialog);
+	accounts_dialog_model_select_first (dialog);
+
+	if (parent) {
+		gtk_window_set_transient_for (GTK_WINDOW (dialog->window),
+					      GTK_WINDOW (parent));
+	}
 
 	gtk_widget_show (dialog->window);
 
-	accounts_dialog_model_select_first (dialog);
-
 	return dialog->window;
 }
 
